@@ -1,102 +1,128 @@
 package Sys::Info::Driver::Linux::OS;
 use strict;
 use vars qw( $VERSION );
-# fstab entries
-use constant FS_SPECIFIER     => 0;
-use constant MOUNT_POINT      => 1;
-use constant FS_TYPE          => 2;
-use constant MOUNT_OPTS       => 3;
-use constant DUMP_FREQ        => 4;
-use constant FS_CHECK_ORDER   => 5;
-# uname()
-use constant SYSNAME          => 0;
-use constant NODENAME         => 1;
-use constant RELEASE          => 2;
-use constant VERSION          => 3;
-use constant MACHINE          => 4;
-# getpwnam()
-use constant REAL_NAME_FIELD  => 6;
-use constant RE_LINUX_VERSION => qr{
-   \A
-   Linux\s+version\s
-   (.+?)
-   \s
-   \(.+?\@.+?\)
-   (.*?)
-   \z
-}xmsi;
 use base qw( Sys::Info::Base );
 use POSIX ();
 use Cwd;
 use Carp qw( croak );
+use Sys::Info::Driver::Linux;
+use Sys::Info::Constants qw( :linux );
 
-$VERSION = '0.50';
+$VERSION = '0.60';
 
 my %OSVERSION; # cache
-my %DISTROFIX = qw( suse SUSE );
-my %PATH      = (
-    fstab   => '/etc/fstab',    # for filesystem type of the current disk
-    uptime  => '/proc/uptime',  # uptime file
-    version => '/proc/version', # os version
-    resolv  => '/etc/resolv.conf',
-    #/etc/issue
-);
 
+my %DISTROFIX = qw( suse SUSE );
+
+my $EDITION   = {
+    #taken from wikipedia
+    ubuntu => {
+        '4.10' => 'Warty Warthog',
+        '5.04' => 'Hoary Hedgehog',
+        '5.10' => 'Breezy Badger',
+        '6.06' => 'Dapper Drake',
+        '6.10' => 'Edgy Eft',
+        '7.04' => 'Feisty Fawn',
+        '7.10' => 'Gutsy Gibbon',
+        '8.04' => 'Hardy Heron',
+        '8.10' => 'Intrepid Ibex',
+    },
+};
+
+my $MANUFACTURER = {
+    #taken from wikipedia
+    ubuntu    => 'Canonical Ltd. / Ubuntu Foundation',
+    centos    => 'Lance Davis',
+    fedora    => 'Fedora Project',
+    debian    => 'Debian Project',
+    mandriva  => 'Mandriva',
+    knoppix   => 'Klaus Knopper',
+    gentoo    => 'Gentoo Foundation',
+    suse      => 'Novell',
+    slackware => 'Patrick Volkerding',
+};
+
+my $EDITION_SUPPORT      = join '|', keys %{ $EDITION      };
+my $MANUFACTURER_SUPPORT = join '|', keys %{ $MANUFACTURER };
+
+# unimplemented
+sub logon_server {}
+
+sub edition {
+    my $self = shift->_populate_osversion;
+    $OSVERSION{RAW}->{EDITION};
+}
+
+sub tz {
+    my $self = shift;
+    return if ! -e proc->{timezone};
+    chomp( my $rv = $self->slurp( proc->{timezone} ) );
+    return $rv;
+}
 
 sub meta {
     my $self = shift;
-    my $id   = shift;
     $self->_populate_osversion();
+
+    my $manufacturer = $OSVERSION{NAME} =~ m{ ($MANUFACTURER_SUPPORT) }xmsi
+                     ? $MANUFACTURER->{ lc $1 }
+                     : undef;
+
+    require POSIX;
+    require Sys::Info::Device;
+
+    my $cpu   = Sys::Info::Device->new('CPU');
+    my $arch  = ($cpu->identify)[0]->{architecture};
+    my %mem   = $self->_parse_meminfo;
+    my @swaps = $self->_parse_swap;
     my %info;
-    $info{manufacturer}              = undef;
+
+    $info{manufacturer}              = $manufacturer;
     $info{build_type}                = undef;
     $info{owner}                     = undef;
     $info{organization}              = undef;
     $info{product_id}                = undef;
     $info{install_date}              = $OSVERSION{RAW}->{BUILD_DATE};
     $info{boot_device}               = undef;
-    $info{time_zone}                 = undef;
-    $info{physical_memory_total}     = undef;
-    $info{physical_memory_available} = undef;
-    $info{page_file_total}           = undef;
-    $info{page_file_available}       = undef;
+
+    $info{physical_memory_total}     = $mem{MemTotal};
+    $info{physical_memory_available} = $mem{MemFree};
+    $info{page_file_total}           = $mem{SwapTotal};
+    $info{page_file_available}       = $mem{SwapFree};
+
     # windows specific
     $info{windows_dir}               = undef;
     $info{system_dir}                = undef;
-    # ????
-    $info{locale}                    = undef;
 
     $info{system_manufacturer}       = undef;
     $info{system_model}              = undef;
-    $info{system_type}               = undef;
-    $info{domain}                    = undef;
+    $info{system_type}               = sprintf "%s based Computer", $arch;
 
-    $info{page_file_path}            = undef;
+    $info{page_file_path}            = join ', ', map { $_->{Filename} } @swaps;
 
-    return %info if ! $id;
-
-    my $lcid = lc $id;
-    if ( ! exists $info{ $lcid } ) {
-        croak "$id meta value is not supported by the underlying Operating System";
-    }
-    return $info{ $lcid };
+    return %info;
 }
 
-sub tz           {}
-sub logon_server {}
-
 sub tick_count {
     my $self = shift;
-    my $uptime = $self->slurp($PATH{uptime}) || return 0;
+    my $uptime = $self->slurp( proc->{uptime} ) || return 0;
     my @uptime = split /\s+/, $uptime;
     # this file has two entries. uptime is the first one. second: idle time
-    return $uptime[0];
+    return $uptime[LIN_UP_TIME];
 }
 
-sub name      { shift->_populate_osversion(); return $OSVERSION{NAME}         }
-sub long_name { shift->_populate_osversion(); return $OSVERSION{LONGNAME}     }
+sub name {
+    my $self = shift->_populate_osversion;
+    my %opt  = @_ % 2 ? () : (@_);
+    my $id   = $opt{long} ? ($opt{edition} ? 'LONGNAME_EDITION' : 'LONGNAME')
+             :              ($opt{edition} ? 'NAME_EDITION'     : 'NAME'    )
+             ;
+    return $OSVERSION{ $id };
+}
+
+
 sub version   { shift->_populate_osversion(); return $OSVERSION{VERSION}      }
-sub build     { shift->_populate_osversion(); return $OSVERSION{RAW}->{BUILD} }
+sub build     { shift->_populate_osversion(); return $OSVERSION{RAW}->{BUILD_DATE} }
 sub uptime    {                               return time - shift->tick_count }
 
 # user methods
@@ -111,23 +137,21 @@ sub is_root {
 }
 
 sub login_name {
-    return '' if defined &Sys::Info::EMULATE;
-    return POSIX::getlogin();
+    my $self  = shift;
+    my %opt   = @_ % 2 ? () : (@_);
+    my $login = POSIX::getlogin() || return;
+    my $rv    = eval { $opt{real} ? (getpwnam $login)[LIN_REAL_NAME_FIELD] : $login };
+    $rv =~ s{ [,]{3,} \z }{}xms if $opt{real};
+    return $rv;
 }
 
-sub login_name_real {
-    my $name = login_name() || return '';
-    my $real = (getpwnam $name)[REAL_NAME_FIELD];
-    return $real || $name;
-}
-
-sub node_name { (POSIX::uname())[NODENAME] }
+sub node_name { (POSIX::uname())[LIN_NODENAME] }
 
 sub domain_name {
     my $self = shift;
     my $domain;
     #hmmmm...
-    foreach my $line ( $self->read_file( $PATH{resolv} ) ) {
+    foreach my $line ( $self->read_file( proc->{resolv} ) ) {
         chomp $line;
         if ( $line =~ m{\A domain \s+ (.*) \z}xmso ) {
             $domain = $1;
@@ -142,15 +166,15 @@ sub fs {
     $self->{current_dir} = Cwd::getcwd();
 
     my(@fstab, @junk, $re);
-    foreach my $line( $self->read_file($PATH{fstab}) ) {
+    foreach my $line( $self->read_file( proc->{fstab} ) ) {
         chomp $line;
         next if $line =~ m[^#];
         @junk = split /\s+/, $line;
         next if ! @junk || @junk != 6;
-        next if lc($junk[FS_TYPE]) eq 'swap'; # ignore swaps
-        $re = $junk[MOUNT_POINT];
+        next if lc($junk[LIN_FS_TYPE]) eq 'swap'; # ignore swaps
+        $re = $junk[LIN_MOUNT_POINT];
         next if $self->{current_dir} !~ m{\Q$re\E}i;
-        push @fstab, [ $re, $junk[FS_TYPE] ];
+        push @fstab, [ $re, $junk[LIN_FS_TYPE] ];
     }
 
     @fstab  = sort( { $b->[0] cmp $a->[0] } @fstab ) if @fstab > 1;
@@ -164,6 +188,34 @@ sub fs {
 
 # ------------------------[ P R I V A T E ]------------------------ #
 
+sub _parse_meminfo {
+    my $self = shift;
+    my %mem;
+    foreach my $line ( split /\n/, $self->slurp( proc->{meminfo} ) ) {
+        chomp $line;
+        my($k, $v) = split /:/, $line;
+        #units in KB
+        $mem{ $k } = (split /\s+/, $self->trim( $v ) )[0];
+    }
+    return %mem;
+}
+
+sub _parse_swap {
+    my $self = shift;
+    my @swaps      = split /\n/, $self->slurp( proc->{swaps} );
+    my @swap_title = split /\s+/, shift( @swaps );
+    my @swap_list;
+    foreach my $line ( @swaps ) {
+        chomp $line;
+        my @data = split /\s+/, $line;
+        push @swap_list,
+            {
+                map { $swap_title[$_] => $data[$_] } 0..$#swap_title
+            };
+    }
+    return @swap_list;
+}
+
 sub _ip {
     my $self = shift;
     my $raw  = qx(ifconfig);
@@ -180,10 +232,10 @@ sub _populate_osversion {
     my $self    = shift;
     my $version = '';
 
-    if (  -e $PATH{'version'} && -f _) {
+    if (  -e proc->{'version'} && -f _) {
         $version =  $self->trim(
                         $self->slurp(
-                            $PATH{'version'},
+                            proc->{'version'},
                             'I can not open linux version file %s for reading: '
                         )
                     );
@@ -197,7 +249,7 @@ sub _populate_osversion {
     # format: 'Linux version 1.2.3 (foo@bar.com)'
     # format: 'Linux version 1.2.3 (foo@bar.com) (gcc 1.2.3)'
     # format: 'Linux version 1.2.3 (foo@bar.com) (gcc 1.2.3 (Redhat blah blah))'
-    if ( $str =~ RE_LINUX_VERSION ) {
+    if ( $str =~ LIN_RE_LINUX_VERSION ) {
         $kernel = $1;
         if ( $distro = $self->trim( $2 ) ) {
             if ( $distro =~ m{ \s\((.+?)\)\) \z }xms ) {
@@ -210,7 +262,7 @@ sub _populate_osversion {
 
     # kernel build date
     $build_date = $self->date2time($build_date) if $build_date;
-    my $build = $build_date ? $self->date2time($build_date) : '';
+    my $build = $build_date || '';
     $build = scalar( localtime $build ) if $build;
 
     require Linux::Distribution;
@@ -222,20 +274,34 @@ sub _populate_osversion {
         $dv  = $linux->distribution_version;
     }
 
+    my $V      = $dv || $kernel;
+    my $osname = $dn || $distro;
+
+    my $edition;
+    if ( $osname =~ m{ ($EDITION_SUPPORT) }xmsi ) {
+        my $id = lc $1;
+        $edition = $EDITION->{ $id }{ $V };
+    }
+
     %OSVERSION = (
-        NAME     => $dn || $distro,
-        LONGNAME => undef,
-        VERSION  => $dv || $kernel,
+        NAME             => $osname,
+        NAME_EDITION     => $edition ? "$osname ($edition)" : $osname,
+        LONGNAME         => '', # will be set below
+        LONGNAME_EDITION => '', # will be set below
+        VERSION  => $V,
         KERNEL   => $kernel,
         RAW      => {
                         BUILD      => defined $build      ? $build      : 0,
                         BUILD_DATE => defined $build_date ? $build_date : 0,
+                        EDITION    => $edition,
                     },
     );
 
-    $OSVERSION{LONGNAME} = sprintf "%s %s (kernel: %s)",
-                                   $OSVERSION{NAME},
-                                   $OSVERSION{VERSION},
+    $OSVERSION{LONGNAME}         = sprintf "%s %s (kernel: %s)",
+                                   @OSVERSION{ qw/ NAME         VERSION / },
+                                   $kernel;
+    $OSVERSION{LONGNAME_EDITION} = sprintf "%s %s (kernel: %s)",
+                                   @OSVERSION{ qw/ NAME_EDITION VERSION / },
                                    $kernel;
     return;
 }
@@ -264,10 +330,6 @@ sub _fs_attributes {
     }->{$fs};
 }
 
-# Sys::Info::EMULATE
-#    I'm emulating linux environment on windows to test module
-#    interface. If this sub is defined, some methods will return false
-
 1;
 
 __END__
@@ -278,11 +340,43 @@ Sys::Info::Driver::Linux::OS - Linux backend
 
 =head1 SYNOPSIS
 
-Nothing public here.
+-
 
 =head1 DESCRIPTION
 
-Nothing public here.
+-
+
+=head1 METHODS
+
+Please see L<Sys::Info::OS> for definitions of these methods and more.
+
+=head2 build
+
+=head2 domain_name
+
+=head2 edition
+
+=head2 fs
+
+=head2 is_root
+
+=head2 login_name
+
+=head2 logon_server
+
+=head2 meta
+
+=head2 name
+
+=head2 node_name
+
+=head2 tick_count
+
+=head2 tz
+
+=head2 uptime
+
+=head2 version
 
 =head1 SEE ALSO
 
